<?php
if ( ! class_exists( 'NJT_ENVATO_AUTO_UPDATE' ) && class_exists( 'NJT_ENVATO_MARKET' ) ) :
	class NJT_ENVATO_AUTO_UPDATE {
		private static $_instance = null;
		public static function instance() {
			if ( is_null( self::$_instance ) ) {
				self::$_instance = new self();
				self::$_instance->njt_add_actions();
			}
			return self::$_instance;
		}
		private function __construct() {
		
		}
		
		public function njt_add_actions() {
			
			// Deferred Download.
			add_action( 'upgrader_package_options', array( $this, 'njt_upgrader_package_down' ), 9 );
			
			add_filter( 'upgrader_pre_download', array( $this, 'njt_pre_download_upgrader' ), 2, 4 );
		}
		public function njt_upgrader_package_down( $options ) {
			$package = $options['package'];
			if ( false !== strrpos( $package, 'deferred_download' ) && false !== strrpos( $package, 'item_id' ) ) {
				parse_str( parse_url( $package, PHP_URL_QUERY ), $vars );
				if ( $vars['item_id'] ) {
					$args               = $this->njt_args_set_bearer( $vars['item_id'] );
					$options['package'] = njt_envato_market()->api()->download( $vars['item_id'], $args );
				}
			}
			return $options;
		}
		public function njt_pre_download_upgrader( $reply, $package, $updater ) {
			if ( strpos( $package, 'marketplace.envato.com/short-dl' ) !== false ) {
				if ( isset( $updater->skin->plugin_info ) && ! empty( $updater->skin->plugin_info['Name'] ) ) {
					$updater->skin->plugin_info['Name'] = $updater->skin->plugin_info['Name'] . '.';
				} else {
					$updater->skin->plugin_info = array(
						'Name' => 'Name',
					);
				}
			}
			return $reply;
		}
		
		public function njt_args_set_bearer( $id ) {
			$token = '';
			$args  = array();
			foreach ( njt_envato_market()->get_option( 'items', array() ) as $item ) {
				if ( absint( $item['id'] ) === absint( $id ) ) {
					$token = $item['token'];
					break;
				}
			}
			if ( ! empty( $token ) ) {
				$args = array(
					'headers' => array(
						'Authorization' => 'Bearer ' . $token,
					),
				);
			}
			return $args;
		}
		/**
		 * Install plugin.
		 *
		 * @since 1.0.0
		 * @codeCoverageIgnore
		 *
		 * @param string $plugin The plugin item ID.
		 */
		public function install_plugin( $plugin ) {
			if ( ! current_user_can( 'install_plugins' ) ) {
				$msg = '
				<div class="wrap">
					<h1>' . __( 'Installing Plugin...', NJT_ABANDONED_CART ) . '</h1>
					<p>' . __( 'You do not have sufficient permissions to install plugins on this site.', NJT_ABANDONED_CART ) . '</p>
					<a href="' . esc_url( 'admin.php?page=' . njt_envato_market()->get_slug() . '&tab=plugins' ) . '">' . __( 'Return to Plugin Installer', NJT_ABANDONED_CART ) . '</a>
				</div>';
				wp_die( $msg );
			}
			check_admin_referer( 'install-plugin_' . $plugin );
			njt_envato_market()->items()->set_plugins( true );
			$install = njt_envato_market()->items()->plugins( 'install' );
			$api     = new stdClass();
			foreach ( $install as $value ) {
				if ( absint( $value['id'] ) === absint( $plugin ) ) {
					$api->name    = $value['name'];
					$api->version = $value['version'];
				}
			}
			$array_api = (array) $api;
			if ( empty( $array_api ) ) {
				$msg = '
				<div class="wrap">
					<h1>' . __( 'Installing Plugin...', NJT_ABANDONED_CART ) . '</h1>
					<p>' . __( 'An error occurred, please check that the item ID is correct.', NJT_ABANDONED_CART ) . '</p>
					<a href="' . esc_url( 'admin.php?page=' . njt_envato_market()->get_slug() . '&tab=plugins' ) . '">' . __( 'Return to Plugin Installer', NJT_ABANDONED_CART ) . '</a>
				</div>';
				wp_die( $msg );
			}
			$title              = sprintf( __( 'Installing Plugin: %s', NJT_ABANDONED_CART ), esc_html( $api->name . ' ' . $api->version ) );
			$nonce              = 'install-plugin_' . $plugin;
			$url                = 'admin.php?page=' . njt_envato_market()->get_slug() . '&action=install-plugin&plugin=' . urlencode( $plugin );
			$type               = 'web'; // Install plugin type, From Web or an Upload.
			$api->download_link = njt_envato_market()->api()->download( $plugin, $this->njt_args_set_bearer( $plugin ) );
			// Must have the upgrader & skin.
			require njt_envato_market()->get_plugin_url() . 'update/class-cartback-upgrader.php';
			require njt_envato_market()->get_plugin_url() . 'update/class-installer-skin.php';
			$upgrader = new CARTBACK_UPGRADER( new CARTBACK_INSTALLER_SKIN( compact( 'title', 'url', 'nonce', 'plugin', 'api' ) ) );
			$upgrader->install( $api->download_link );
		}
	}
endif;
