<?php
/**
 * CARTBACK_UPDATE_ENVATO API class.
 */
if ( ! class_exists( 'CARTBACK_UPDATE_ENVATO' ) && class_exists( 'NJT_ENVATO_MARKET' ) ) :
	
	class CARTBACK_UPDATE_ENVATO {
		private static $_instance = null;
		/**
		 * The Envato API personal token.
		 */
		public $token;
		
		public static function instance() {
			if ( is_null( self::$_instance ) ) {
				self::$_instance = new self();
				self::$_instance->init_globals();
			}
			return self::$_instance;
		}
		private function __construct() {
		
		}
		/**
		 * Setup the class globals.
		 *
		 */
		private function init_globals() {
			// Envato API token.k7InJWudx9WRF9cDwSOQ25cIOQKPkrTx
			$this->token =get_option('cartback_personal_token');//"k7InJWudx9WRF9cDwSOQ25cIOQKPkrTx";//"OpDbVEevcGfoLZ8SZsIh2NdXBs3T3KDX";//get_option('cb_personal_token');
		}
		public function get_personal_token(){
			return $this->token;
		}
		public function request( $url, $args = array() ) {
			$defaults = array(
				'headers' => array(
					'Authorization' => 'Bearer ' . $this->token,
					'User-Agent'    => 'WordPress - Envato Market ' . njt_envato_market()->get_version(),
				),
				'timeout' => 14,
			);
			$args     = wp_parse_args( $args, $defaults );
			$token = trim( str_replace( 'Bearer', '', $args['headers']['Authorization'] ) );
			if ( empty( $token ) ) {
				return new WP_Error( 'api_token_error', __( 'An API token is required.', NJT_ABANDONED_CART ) );
			}
			// Make an API request.
			$response = wp_remote_get( esc_url_raw( $url ), $args );
			// Check the response code.
			$response_code    = wp_remote_retrieve_response_code( $response );
			$response_message = wp_remote_retrieve_response_message( $response );
			if ( ! empty( $response->errors ) && isset( $response->errors['http_request_failed'] ) ) {
				// API connectivity issue, inject notice into transient with more details.
				$option = njt_envato_market()->get_options();
				if ( empty( $option['notices'] ) ) {
					$option['notices'] = [];
				}
				$option['notices']['http_error'] = current( $response->errors['http_request_failed'] );
				njt_envato_market()->set_options( $option );
				return new WP_Error( 'http_error', esc_html( current( $response->errors['http_request_failed'] ) ) );
			}
			if ( 200 !== $response_code && ! empty( $response_message ) ) {
				return new WP_Error( $response_code, $response_message );
			} elseif ( 200 !== $response_code ) {
				return new WP_Error( $response_code, __( 'An unknown API error occurred.', NJT_ABANDONED_CART ) );
			} else {
				$return = json_decode( wp_remote_retrieve_body( $response ), true );
				if ( null === $return ) {
					return new WP_Error( 'api_error', __( 'An unknown API error occurred.', NJT_ABANDONED_CART ) );
				}
				return $return;
			}
		}
		
		public function deferred_download( $id ) {
			if ( empty( $id ) ) {
				return '';
			}
			$args = array(
				'deferred_download' => true,
				'item_id'           => $id,
			);
			return add_query_arg( $args, esc_url( njt_envato_market()->get_page_url() ) );
		}
		
		public function download( $id, $args = array() ) {
			if ( empty( $id ) ) {
				return false;
			}
			$url      = 'https://api.envato.com/v2/market/buyer/download?item_id=' . $id . '&shorten_url=true';
			$response = $this->request( $url, $args );
			// @todo Find out which errors could be returned & handle them in the UI.
			if ( is_wp_error( $response ) || empty( $response ) || ! empty( $response['error'] ) ) {
				return false;
			}
			if ( ! empty( $response['wordpress_theme'] ) ) {
				return $response['wordpress_theme'];
			}
			if ( ! empty( $response['wordpress_plugin'] ) ) {
				return $response['wordpress_plugin'];
			}
			// Missing a WordPress theme and plugin, report an error.
			$option = njt_envato_market()->get_options();
			if ( ! isset( $option['notices'] ) ) {
				$option['notices'] = [];
			}
			$option['notices']['missing-package-zip'] = true;
			njt_envato_market()->set_options( $option );
			return false;
		}
		
		public function item( $id, $args = array() ) {
			$url      = 'https://api.envato.com/v2/market/catalog/item?id=' . $id;
			$response = $this->request( $url, $args );
			if ( is_wp_error( $response ) || empty( $response ) ) {
				return false;
			}
			if ( ! empty( $response['wordpress_theme_metadata'] ) ) {
				return $this->normalize_theme( $response );
			}
			if ( ! empty( $response['wordpress_plugin_metadata'] ) ) {
				return $this->normalize_plugin( $response );
			}
			return false;
		}
		
		public function plugins( $args = array() ) {
			$plugins = array();
			$url      = 'https://api.envato.com/v2/market/buyer/list-purchases?filter_by=wordpress-plugins';
			$response = $this->request( $url, $args );
			if ( is_wp_error( $response ) || empty( $response ) || empty( $response['results'] ) ) {
				return $plugins;
			}
			foreach ( $response['results'] as $plugin ) {
				$plugins[] = $this->normalize_plugin( $plugin['item'] );
			}
			return $plugins;
		}
		//
		public function get_list_plugin_user_buy(){
			$url      = 'https://api.envato.com/v3/market/buyer/list-purchases?filter_by=wordpress-plugins';
			$headers = array('Authorization: Bearer '.$this->token);
		    $ch = curl_init();
		    curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
		    curl_setopt($ch, CURLOPT_URL, $url);
		    curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 10);
		    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
		    $str = curl_exec($ch);
		    curl_close($ch);
    		return json_decode($str,true);
		}
		public function get_purchase_code_with_product_id(){
			$list_plugin_buy = $this->get_list_plugin_user_buy();
			if(isset($list_plugin_buy["results"]) && $list_plugin_buy["count"] > 0){
				foreach ($list_plugin_buy["results"] as $key => $plugin) {
					if($plugin["item"]["id"] == "20852369"){
						$purchage_code = $plugin['code'];
						break;
					}
				}
				if(isset($purchage_code))
					return $purchage_code;
				return false;
			}
			return false;
		}
		
		public function normalize_plugin( $plugin ) {
			$requires = null;
			$tested   = null;
			$versions = array();
			// Set the required and tested WordPress version numbers.
			foreach ( $plugin['attributes'] as $k => $v ) {
				if ( ! empty( $v['name'] ) && 'compatible-software' === $v['name'] && ! empty( $v['value'] ) && is_array( $v['value'] ) ) {
					foreach ( $v['value'] as $version ) {
						$versions[] = str_replace( 'WordPress ', '', trim( $version ) );
					}
					if ( ! empty( $versions ) ) {
						$requires = $versions[ count( $versions ) - 1 ];
						$tested   = $versions[0];
					}
					break;
				}
			}
			$plugin_normalized = array(
				'id'              => $plugin['id'],
				'name'            => ( ! empty( $plugin['wordpress_plugin_metadata']['plugin_name'] ) ? $plugin['wordpress_plugin_metadata']['plugin_name'] : '' ),
				'author'          => ( ! empty( $plugin['wordpress_plugin_metadata']['author'] ) ? $plugin['wordpress_plugin_metadata']['author'] : '' ),
				'version'         => ( ! empty( $plugin['wordpress_plugin_metadata']['version'] ) ? $plugin['wordpress_plugin_metadata']['version'] : '' ),
				'description'     => self::remove_non_unicode( strip_tags( $plugin['wordpress_plugin_metadata']['description'] ) ),
				'url'             => ( ! empty( $plugin['url'] ) ? $plugin['url'] : '' ),
				'author_url'      => ( ! empty( $plugin['author_url'] ) ? $plugin['author_url'] : '' ),
				'thumbnail_url'   => ( ! empty( $plugin['thumbnail_url'] ) ? $plugin['thumbnail_url'] : '' ),
				'landscape_url'   => ( ! empty( $plugin['previews']['landscape_preview']['landscape_url'] ) ? $plugin['previews']['landscape_preview']['landscape_url'] : '' ),
				'requires'        => $requires,
				'tested'          => $tested,
				'number_of_sales' => ( ! empty( $plugin['number_of_sales'] ) ? $plugin['number_of_sales'] : '' ),
				'updated_at'      => ( ! empty( $plugin['updated_at'] ) ? $plugin['updated_at'] : '' ),
				'rating'          => ( ! empty( $plugin['rating'] ) ? $plugin['rating'] : '' ),
			);
			// No main thumbnail in API response, so we grab it from the preview array.
			if ( empty( $plugin_normalized['landscape_url'] ) && ! empty( $plugin['previews'] ) && is_array( $plugin['previews'] ) ) {
				foreach ( $plugin['previews'] as $possible_preview ) {
					if ( ! empty( $possible_preview['landscape_url'] ) ) {
						$plugin_normalized['landscape_url'] = $possible_preview['landscape_url'];
						break;
					}
				}
			}
			if ( empty( $plugin_normalized['thumbnail_url'] ) && ! empty( $plugin['previews'] ) && is_array( $plugin['previews'] ) ) {
				foreach ( $plugin['previews'] as $possible_preview ) {
					if ( ! empty( $possible_preview['icon_url'] ) ) {
						$plugin_normalized['thumbnail_url'] = $possible_preview['icon_url'];
						break;
					}
				}
			}
			return $plugin_normalized;
		}
		
		static private function remove_non_unicode( $retval ) {
			return preg_replace( '/[\x00-\x1F\x80-\xFF]/', '', $retval );
		}
	}
endif;
