<?php
/**
 * Items class.
 *
 * @package Envato_Market
 */
if ( ! class_exists( 'CARTBACK_UPDATE_ITEMS' ) ) :
	
	class CARTBACK_UPDATE_ITEMS {
		
		private static $_instance = null;
		
		private static $plugins = array();
		
		private static $wp_plugins = array();
		
		public static function instance() {
			if ( is_null( self::$_instance ) ) {
				self::$_instance = new self();
				self::$_instance->init_actions();
			}
			return self::$_instance;
		}
		
		private function __construct() {
		
		}
		
		public function init_actions() {
			// Check for theme & plugin updates.
			add_filter( 'http_request_args', array( $this, 'update_check' ), 5, 2 );
			// Inject plugin updates into the response array.
			add_filter( 'pre_set_site_transient_update_plugins', array( $this, 'update_plugins' ), 5, 1 );
			add_filter( 'pre_set_transient_update_plugins', array( $this, 'update_plugins' ), 5, 1 );
			// Inject plugin information into the API calls.
			add_filter( 'plugins_api', array( $this, 'plugins_api' ), 10, 3 );
			// Rebuild the saved plugin data.
			add_action( 'activated_plugin', array( $this, 'rebuild_plugins' ) );
			add_action( 'deactivated_plugin', array( $this, 'rebuild_plugins' ) );
		}
		
		public function plugins( $group = '' ) {
			if ( ! empty( $group ) ) {
				if ( isset( self::$plugins[ $group ] ) ) {
					return self::$plugins[ $group ];
				} else {
					return array();
				}
			}
			return self::$plugins;
		}
		
		public function wp_plugins( $flush = false ) {
			if ( empty( self::$wp_plugins ) || true === $flush ) {
				wp_cache_flush();
				self::$wp_plugins = get_plugins();
			}
			return self::$wp_plugins;
		}
		
		public function update_check( $request, $url ) {
			// Plugin update request.
			if ( false !== strpos( $url, '//api.wordpress.org/plugins/update-check/1.1/' ) ) {
				/**
				 * Excluded theme slugs that should never ping the WordPress API.
				 * We don't need the extra http requests for themes we know are premium.
				 */
				self::set_plugins();
				$installed = self::$plugins['installed'];
				// Decode JSON so we can manipulate the array.
				$data = json_decode( $request['body']['plugins'] );
				// Remove the excluded themes.
				foreach ( $installed as $slug => $id ) {
					unset( $data->plugins->$slug );
				}
				// Encode back into JSON and update the response.
				$request['body']['plugins'] = wp_json_encode( $data );
			}
			return $request;
		}
		
		public function update_plugins( $transient ) {
			$cb_product_id = NJT_ABANDONED_ENVATO_PRODUCT_ID;
			self::set_plugins( true );
			// Process premium plugin updates.
			$installed = array_merge( self::$plugins['active'], self::$plugins['installed'] );
			$plugins   = self::wp_plugins();
			foreach ( $installed as $plugin => $premium ) {
				if($premium['id'] == $cb_product_id){
					if ( isset( $plugins[ $plugin ] ) && version_compare( $plugins[ $plugin ]['Version'], $premium['version'], '<' ) ) {
						$_plugin                        = array(
							'slug'        => dirname( $plugin ),
							'plugin'      => $plugin,
							'new_version' => $premium['version'],
							'url'         => $premium['url'],
							'package'     => njt_envato_market()->api()->deferred_download( $premium['id'] ),
						);
						$transient->response[ $plugin ] = (object) $_plugin;
					}
				}
			}
			return $transient;
		}
		
		public function plugins_api( $response, $action, $args ) {
			self::set_plugins( true );
			// Process premium theme updates.
			if ( 'plugin_information' === $action && isset( $args->slug ) ) {
				$installed = array_merge( self::$plugins['active'], self::$plugins['installed'] );
				foreach ( $installed as $slug => $plugin ) {
					if ( dirname( $slug ) === $args->slug ) {
						$response                 = new stdClass();
						$response->slug           = $args->slug;
						$response->plugin         = $slug;
						$response->plugin_name    = $plugin['name'];
						$response->name           = $plugin['name'];
						$response->version        = $plugin['version'];
						$response->author         = $plugin['author'];
						$response->homepage       = $plugin['url'];
						$response->requires       = $plugin['requires'];
						$response->tested         = $plugin['tested'];
						$response->downloaded     = $plugin['number_of_sales'];
						$response->last_updated   = $plugin['updated_at'];
						$response->sections       = array( 'description' => $plugin['description'] );
						$response->banners['low'] = $plugin['landscape_url'];
						$response->rating         = ! empty( $plugin['rating'] ) && ! empty( $plugin['rating']['rating'] ) ? $plugin['rating']['rating'] / 5 * 100 : 0;
						$response->num_ratings    = ! empty( $plugin['rating'] ) && ! empty( $plugin['rating']['count'] ) ? $plugin['rating']['count'] : 0;
						$response->download_link  = njt_envato_market()->api()->deferred_download( $plugin['id'] );
						break;
					}
				}
			}
			return $response;
		}
		
		public function set_plugins( $forced = false, $use_cache = false, $args = array() ) {
			self::$plugins = get_site_transient( njt_envato_market()->get_option_name() . '_plugins' );
			$plugins = njt_envato_market()->api()->plugins();
			$personal_token_new = njt_envato_market()->api()->get_personal_token();
			if(empty($plugins)){
				self::$plugins = false;
			}else{
				$token_cached = get_option("cache_personal_token_envato");
				if($personal_token_new != $token_cached){
					self::$plugins = false;
				}
			}
			if ( false === self::$plugins || true === $forced ) {
				update_option("cache_personal_token_envato",$personal_token_new);
				foreach ( njt_envato_market()->get_option( 'items', array() ) as $item ) {
					if ( empty( $item ) ) {
						continue;
					}
					if ( 'plugin' === $item['type'] ) {
						$request_args = array(
							'headers' => array(
								'Authorization' => 'Bearer ' . $item['token'],
							),
						);
						$request      = njt_envato_market()->api()->item( $item['id'], $request_args );
						if ( false !== $request ) {
							$plugins[] = $request;
						}
					}
				}
				self::process_plugins( $plugins, $args );
			} elseif ( true === $use_cache ) {
				self::process_plugins( self::$plugins['purchased'], $args );
			}
		}
		
		public function rebuild_plugins( $plugin ) {
			$remove = ( 'deactivated_plugin' === current_filter() ) ? true : false;
			self::set_plugins(
				false, true, array(
					'plugin' => $plugin,
					'remove' => $remove,
				)
			);
		}
		/**
		 * Normalizes a string to do a value check against.
		 *
		 */
		public function normalize( $string ) {
			return strtolower( html_entity_decode( wp_strip_all_tags( $string ) ) );
		}
		//
		public function check_token_permissions($id_product) {
			$flag = 0;
			$plugins = njt_envato_market()->api()->plugins();
			if(empty($plugins))
				return false;
			foreach ($plugins as $key => $plugin) {
				if($plugin['id'] == $id_product){
					$flag = 1;
					break;
				}
			}
			if($flag == 1)
				return true;
			return false;
		}
		
		private function process_plugins( $purchased, $args = array() ) {
			if ( is_wp_error( $purchased ) ) {
				$purchased = array();
			}
			$active    = array();
			$installed = array();
			$install   = $purchased;
			if ( ! empty( $purchased ) ) {
				foreach ( self::wp_plugins( true ) as $slug => $plugin ) {
					foreach ( $install as $key => $value ) {
						if ( $this->normalize( $value['name'] ) === $this->normalize( $plugin['Name'] ) && $this->normalize( $value['author'] ) === $this->normalize( $plugin['Author'] ) && file_exists( WP_PLUGIN_DIR . '/' . $slug ) ) {
							$installed[ $slug ] = $value;
							unset( $install[ $key ] );
						}
					}
				}
			}
			foreach ( $installed as $slug => $plugin ) {
				$condition = false;
				if ( ! empty( $args ) && $slug === $args['plugin'] ) {
					if ( true === $args['remove'] ) {
						continue;
					}
					$condition = true;
				}
				if ( $condition || is_plugin_active( $slug ) ) {
					$active[ $slug ] = $plugin;
					unset( $installed[ $slug ] );
				}
			}
			self::$plugins['purchased'] = array_unique( $purchased, SORT_REGULAR );
			self::$plugins['active']    = array_unique( $active, SORT_REGULAR );
			self::$plugins['installed'] = array_unique( $installed, SORT_REGULAR );
			self::$plugins['install']   = array_unique( array_values( $install ), SORT_REGULAR );
			set_site_transient( njt_envato_market()->get_option_name() . '_plugins', self::$plugins, HOUR_IN_SECONDS );
		}
	}
	//
endif;
