<?php
/**
 * Envato Market class.
 *
 * @package NJT_ENVATO_MARKET
 */
if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}
if ( ! class_exists( 'NJT_ENVATO_MARKET' ) ) :
	
	final class NJT_ENVATO_MARKET {
		
		private static $_instance = null;
		
		private $data;
		
		private $slug;
		
		private $version;
		
		private $plugin_url;
		
		private $plugin_path = NJT_ABANDONED_CART_DIR;
		private $page_url;
		
		private $option_name;
		
		public static function instance() {
			if ( is_null( self::$_instance ) ) {
				self::$_instance = new self();
				self::$_instance->init_globals();
				self::$_instance->init_includes();
				self::$_instance->init_actions();
			}
			return self::$_instance;
		}
		private function init_globals() {
			$this->data        = new stdClass();
			$this->version     = NJT_ABANDONED_CART_VERSION;
			$this->slug        = 'njt-abandoned-cart-settings';
			$this->option_name = self::sanitize_key( $this->slug );
			$this->plugin_url  = NJT_ABANDONED_CART_URL;
			$this->plugin_path = NJT_ABANDONED_CART_DIR;
			$this->page_url    = admin_url( 'admin.php?page=' . $this->slug );
		}
		
		private function __construct() {
		
		}
		private function init_includes() {
			require $this->plugin_path . '/cartback-update/classess/class-njt-auto-update.php';
			require $this->plugin_path . '/cartback-update/classess/class-njt-envato-api.php';
			require $this->plugin_path . '/cartback-update/classess/class-njt-envato-items.php';
			
			if ( ! function_exists( 'njt_cartback_update' ) ) :
				function njt_cartback_update() {
					return NJT_ENVATO_AUTO_UPDATE::instance();
				}
			endif;
			njt_cartback_update();
		}
		private function init_actions() {
			// Load Upgrader.
			add_action( 'init', array( $this, 'items' ) );
		}
		public function get_slug() {
			return $this->slug;
		}
		public function get_plugin_url() {
			return $this->plugin_url;
		}
		public function get_page_url() {
			return $this->page_url;
		}
		
		private function sanitize_key( $key ) {
			return preg_replace( '/[^A-Za-z0-9\_]/i', '', str_replace( array( '-', ':' ), '_', $key ) );
		}
		
		private function convert_data( $array ) {
			foreach ( (array) $array as $key => $value ) {
				if ( is_array( $value ) ) {
					$array[ $key ] = self::convert_data( $value );
				}
			}
			return (object) $array;
		}
		public function get_option_name() {
			return $this->option_name;
		}
		public function set_option( $name, $option ) {
			$options          = self::get_options();
			$name             = self::sanitize_key( $name );
			$options[ $name ] = esc_html( $option );
			$this->set_options( $options );
		}
		public function set_options( $options ) {
			update_option( $this->option_name, $options );
		}
		public function get_option( $name, $default = '' ) {
			$options = self::get_options();
			$name    = self::sanitize_key( $name );
			return isset( $options[ $name ] ) ? $options[ $name ] : $default;
		}
		public function get_options() {
			return get_option( $this->option_name, array() );
		}
		
		public function get_version() {
			return $this->version;
		}
		public function api() {
			return CARTBACK_UPDATE_ENVATO::instance();
		}
		
		public function items() {
			return CARTBACK_UPDATE_ITEMS::instance();
		}
	}
endif;
